@@ -88,11 +88,6 @@ FlatpakSourcesBackend::FlatpakSourcesBackend(const QVector<FlatpakInstallation *
     connect(m_flathubAction, &DiscoverAction::triggered, this, [this]() {
         addSource(QStringLiteral("https://flathub.org/repo/flathub.flatpakrepo"));
     });
-    for (auto installation : installations) {
-        if (!listRepositories(installation)) {
-            qWarning() << "Failed to list repositories from installation" << installation;
-        }
-    }
 
     m_noSourcesItem->setEnabled(false);
     if (m_sources->rowCount() == 0) {
@@ -283,81 +278,11 @@ QVariantList FlatpakSourcesBackend::actions() const
     return {QVariant::fromValue<QObject *>(m_flathubAction)};
 }
 
-bool FlatpakSourcesBackend::listRepositories(FlatpakInstallation *installation)
-{
-    Q_ASSERT(installation);
-
-    g_autoptr(GCancellable) cancellable = g_cancellable_new();
-    g_autoptr(GPtrArray) remotes = flatpak_installation_list_remotes(installation, cancellable, nullptr);
-
-    if (!remotes) {
-        return false;
-    }
-
-    for (uint i = 0; i < remotes->len; i++) {
-        FlatpakRemote *remote = FLATPAK_REMOTE(g_ptr_array_index(remotes, i));
-
-        if (flatpak_remote_get_noenumerate(remote)) {
-            continue;
-        }
-
-        addRemote(remote, installation);
-    }
-
-    return true;
-}
-
-void FlatpakSourcesBackend::populateRemote(FlatpakRemote *remote, const QString &name, const QString &url, const QString &gpgKey)
-{
-    flatpak_remote_set_url(remote, url.toUtf8().constData());
-    flatpak_remote_set_noenumerate(remote, false);
-    flatpak_remote_set_title(remote, name.toUtf8().constData());
-
-    if (!gpgKey.isEmpty()) {
-        gsize dataLen = 0;
-        g_autofree guchar *data = nullptr;
-        g_autoptr(GBytes) bytes = nullptr;
-        data = g_base64_decode(gpgKey.toUtf8().constData(), &dataLen);
-        bytes = g_bytes_new(data, dataLen);
-        flatpak_remote_set_gpg_verify(remote, true);
-        flatpak_remote_set_gpg_key(remote, bytes);
-    } else {
-        flatpak_remote_set_gpg_verify(remote, false);
-    }
-}
-
-FlatpakRemote *FlatpakSourcesBackend::installSource(FlatpakResource *resource)
-{
-    g_autoptr(GCancellable) cancellable = g_cancellable_new();
-
-    auto remote = flatpak_installation_get_remote_by_name(m_preferredInstallation, resource->flatpakName().toUtf8().constData(), cancellable, nullptr);
-    if (remote) {
-        qWarning() << "Source " << resource->flatpakName() << " already exists in" << flatpak_installation_get_path(m_preferredInstallation);
-        return nullptr;
-    }
-
-    remote = flatpak_remote_new(resource->flatpakName().toUtf8().constData());
-    populateRemote(remote,
-                   resource->comment(),
-                   resource->getMetadata(QStringLiteral("repo-url")).toString(),
-                   resource->getMetadata(QStringLiteral("gpg-key")).toString());
-    if (!resource->branch().isEmpty()) {
-        flatpak_remote_set_default_branch(remote, resource->branch().toUtf8().constData());
-    }
-
-    g_autoptr(GError) error = nullptr;
-    if (!flatpak_installation_modify_remote(m_preferredInstallation, remote, cancellable, &error)) {
-        qWarning() << "Failed to add source " << resource->flatpakName() << error->message;
-        return nullptr;
-    }
-
-    addRemote(remote, m_preferredInstallation);
-
-    return remote;
-}
-
 void FlatpakSourcesBackend::addRemote(FlatpakRemote *remote, FlatpakInstallation *installation)
 {
+    if (flatpak_remote_get_noenumerate(remote)) {
+        return;
+    }
     const QString id = QString::fromUtf8(flatpak_remote_get_name(remote));
     const QString title = QString::fromUtf8(flatpak_remote_get_title(remote));
     const QUrl remoteUrl(QString::fromUtf8(flatpak_remote_get_url(remote)));
@@ -376,11 +301,21 @@ void FlatpakSourcesBackend::addRemote(FlatpakRemote *remote, FlatpakInstallation
         label = i18n("%1 (user)", label);
     }
 
+    for (int i = 0, c = m_sources->rowCount(); i < c; ++i) {
+        FlatpakSourceItem *item = static_cast<FlatpakSourceItem *>(m_sources->item(i));
+        if (item->data(Qt::StatusTipRole) == remoteUrl && item->flatpakInstallation() == installation) {
+            qDebug() << "we already have an item for this" << remoteUrl;
+            return;
+        }
+    }
+
     FlatpakBackend *backend = qobject_cast<FlatpakBackend *>(parent());
     FlatpakSourceItem *it = new FlatpakSourceItem(label, remote, backend);
+    const int prio = flatpak_remote_get_prio(remote);
     it->setData(remoteUrl.isLocalFile() ? remoteUrl.toLocalFile() : remoteUrl.host(), Qt::ToolTipRole);
     it->setData(remoteUrl, Qt::StatusTipRole);
     it->setData(id, IdRole);
+    it->setData(prio, PrioRole);
     it->setCheckState(flatpak_remote_get_disabled(remote) ? Qt::Unchecked : Qt::Checked);
 #if FLATPAK_CHECK_VERSION(1, 4, 0)
     it->setData(QString::fromUtf8(flatpak_remote_get_icon(remote)), IconUrlRole);
@@ -388,24 +323,22 @@ void FlatpakSourcesBackend::addRemote(FlatpakRemote *remote, FlatpakInstallation
     it->setCheckable(true);
     it->setFlatpakInstallation(installation);
 
-    int idx = -1;
-    {
-        const auto conf = KSharedConfig::openConfig();
-        const KConfigGroup group = conf->group("FlatpakSources");
-        const auto ids = group.readEntry<QStringList>("Sources", QStringList());
+    // Add the remotes before those with lower priorities, after the rest.
+    // We disambiguate with internal discover settings
+    const auto conf = KSharedConfig::openConfig();
+    const KConfigGroup group = conf->group("FlatpakSources");
+    const auto ids = group.readEntry<QStringList>("Sources", QStringList());
+    const int ourIdx = ids.indexOf(id);
 
-        const auto ourIdx = ids.indexOf(id);
-        if (ourIdx < 0) { // If not present, we put it on top
-            idx = 0;
-        } else {
-            idx = 0;
-            for (int c = m_sources->rowCount(); idx < c; ++idx) {
-                const auto compIt = m_sources->item(idx);
-                const int compIdx = ids.indexOf(compIt->data(IdRole).toString());
-                if (compIdx >= ourIdx) {
-                    break;
-                }
-            }
+    int idx, c;
+    for (c = m_sources->rowCount(), idx = c; idx < c; ++idx) {
+        const auto compIt = m_sources->item(idx);
+        if (prio > compIt->data(PrioRole).toInt()) {
+            break;
+        }
+        const int compIdx = ids.indexOf(compIt->data(IdRole).toString());
+        if (compIdx >= ourIdx) {
+            break;
         }
     }
 
